"""CSV Generator"""
from typing import Optional
import csv
import faker
from data_genie.utils import generate_file_name, get_file_location
from data_genie.faker_utils import callable_methods, random_faker_method

fake = faker.Faker()


def generate_data(rows: int, columns: int) -> list[list[str]]:
    """Generates data for the CSV file"""
    keys = []
    for _ in range(columns):
        keys.append(random_faker_method())
    data = [keys]
    for _ in range(rows):
        data.append(callable_methods(keys))
    return data


def generate_csv(rows: int,
                 columns: int,
                 *,
                 file_location: Optional[str] = None,
                 file_name: Optional[str] = None,
                 preview: bool = False,
                 stream: bool = False,
                 ):
    """returns a csv file"""
    data = generate_data(rows, columns)

    # safe check on file location
    file_location = get_file_location(location=file_location)

    # safe check on file name
    file_name = generate_file_name(file_name=file_name, file_type='csv')

    if preview:
        counter = 0
        for row in data:
            print(row)
            counter += 1
            if counter == 5:
                break
    elif stream:
        # handle data stream
        pass
    else:
        full_file_name = f'{file_location}/{file_name}'
        try:
            with open(full_file_name, 'w', newline='', encoding='utf-8') as csvfile:
                writer = csv.writer(csvfile)
                writer.writerows(data)
        except FileNotFoundError as file_not_found_error:
            raise ValueError(f'Error: {full_file_name} does not exist') \
                from file_not_found_error
        except PermissionError as permission_error:
            raise ValueError(f'Error: Insufficient permission to write to'
                             f' {full_file_name}') from permission_error
        except IOError as io_error:
            raise ValueError(f'Error: I/O error occurred when writing to'
                             f' {full_file_name}') from io_error


def data_stream(data):
    """generator function that yields one row at a time"""
    for row in data:
        yield row



"""Utils"""
from typing import Optional
import os
import datetime
import logging


def get_file_location(location: Optional[str] = None):
    """return file location"""
    if location is None:
        location = os.environ.get('DATAGENIE_DIR')
    if location is None:
        location = '.'
    location = os.path.abspath(location)
    if not os.path.exists(location):
        logger(f'{location} does not exists', error=True)
        return None
    if not os.path.isdir(location):
        logger(f'{location} is not a directory', error=True)
        return None
    return location


def generate_file_name(file_type: str, file_name: Optional[str] = None,
                       date_formatter: str = '%Y-%m-%d-%H-%M-%S'):
    """generates file name for datasets"""
    if file_name:
        # Strip file extension from file_name if it is already included
        file_name = file_name.rsplit('.', 1)[0]
    else:
        now = datetime.datetime.now()
        formatted_date = now.strftime(date_formatter)
        file_name = f'datagenie_{formatted_date}'
    return f'{file_name}.{file_type}'


def get_datagenie_env():
    """returns all the env variables related to datagenie"""
    env = {}
    for key, value in os.environ.items():
        if key.startswith('DATAGENIE_'):
            env[key] = value
    return env


def logger(value, *, debug=True, info=False, error=False):
    """ logger for datagenie"""
    if debug:
        logging.getLogger().setLevel(logging.DEBUG)
    if info:
        logging.getLogger().setLevel(logging.INFO)
        logging.info('%s', value)
    if error:
        logging.getLogger().setLevel(logging.ERROR)
        logging.error('%s', value)



"""Faker Utils"""
import inspect
import secrets
import logging
from faker import Faker

fake = Faker('en_US')


def get_callable_method_names() -> list[str]:
    """Returns all the available methods from faker object"""

    # exceptional methods are non-callable methods or required some params
    exceptional_list = ['cache_pattern', 'factories', 'generator_attrs',
                        'seed', 'locales', 'providers', 'random',
                        'unique', 'weights']
    # filtering methods
    methods = [obj for obj in dir(fake)
               if not obj.startswith('__') and not obj.startswith('_')]
    filtered_list = [x for x in methods if x not in exceptional_list]
    callable_methods_names = []

    # checking if there are any positional params required
    for method in filtered_list:
        try:
            method_to_call = getattr(fake, method)
            sig = inspect.signature(method_to_call)
            if not sig.parameters:
                callable_methods_names.append(method)
        except AttributeError as attribute_error:
            logging.error('Error getting method %s: %s', method, attribute_error)
        # except Exception as error:
        #     logging.error('Error getting method %s: %s', method, error)
    return callable_methods_names


def random_faker_method():
    """Random function from faker"""
    methods = get_callable_method_names()
    method = secrets.choice(methods)
    return method


def callable_methods(methods):
    """returns list of callable faker methods from method names"""
    call_methods = []
    for method in methods:
        method_to_call = getattr(fake, method)
        call_methods.append(method_to_call())
    return call_methods



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['datasets_genie']

package_data = \
{'': ['*']}

install_requires = \
['faker>=15.3.4,<16.0.0']

setup_kwargs = {
    'name': 'datasets-genie',
    'version': '0.1.0',
    'description': 'Package for generating fake data in CSV, JSON, and XML formats. It can be used to create placeholder data for testing or demonstration purposes.',
    'long_description': '# datagenie\n\npackage for generating fake data in CSV, JSON, and XML formats. It can be used to create placeholder data for testing or demonstration purposes.\n\n```\npip install datagenie\n```\n\n',
    'author': 'Sai Ranga Reddy Nukala',
    'author_email': 'sairangareddy22@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.9,<4.0',
}


setup(**setup_kwargs)
